unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses DeltaBlue;

{$R *.dfm}
const StrengthName : array[TStrength] of string[13] = (
  'stRequired', 'stStrongPreferred', 'stPreferred', 'stStrongDefault',
  'stDefault', 'stWeakDefault', 'stWeakest' );

function Variable_Print( V : TConstraintVariable ) : string;
begin
  Result := Format( '%s( %s : "%s" ) ', [V.Name, StrengthName[V.WalkStrength],
                        V.Value ] );
end;

function Constraint_Print( C : TConstraint ) : string;
var
  outIndex, I : integer;
begin
  if not C.Satisfied then
  begin
    Result := 'Unsatisfied '+ StrengthName[C.Strength]+ ' ' + C.ClassName + '( '  ;
    for I := 0 to C.Variables.Count-1 do
    begin
      Result := Result + Variable_Print( C.Variables[I] );
    end;
    Result := Result + ')';
  end
  else
  begin
    outIndex := C.OutputMap[C.SelectedMethod];
    Result := 'Satisfied '+ StrengthName[C.Strength] + ' ' + C.ClassName + '( ';
    for I := 0 to C.Variables.Count-1 do
    begin
      if ( I <> outIndex ) then
        Result := Result + Variable_Print( C.Variables[I] );
    end;
    Result := Result + ' -> ' + Variable_Print( C.Variables[OutIndex] );
    Result := Result + ')';
  end;
end;

procedure TempConverter( Strings : TStrings );
var
  celcius, fahrenheit, t1, t2, nine, five, thirtyTwo : TConstraintVariable;
  addC, multC1, multC2 : TConstraint;
  engine : TConstraintEngine;
begin
  Engine := TConstraintEngine.Create;
  try
    celcius := Engine.CreateVariable( 'C', 0 );
    fahrenheit := Engine.CreateVariable( 'F', 0 );
    t1 := Engine.CreateVariable( 't1', 1 );
    t2 := Engine.CreateVariable( 't2', 1 );
    five := Engine.CreateConstant( '5', 5 );
    nine := Engine.CreateConstant( '9', 9 );
    thirtyTwo := Engine.CreateConstant( '32', 32 );

    Strings.Clear;
    Strings.Add( 'Before adding constraints:' );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );

    Strings.Add( 'After adding constraints:' );
    multC1 := engine.AddConstraint( TMultiplyConstraint.Create( celcius, nine, t1, stRequired ) );
    multC2 := engine.AddConstraint( TMultiplyConstraint.Create( t2, five, t1, stRequired ));
    addC := engine.AddConstraint( TSumConstraint.Create( t2, thirtyTwo, fahrenheit, stRequired ));

    Strings.Add( Constraint_Print( multC1 ) );
    Strings.Add( Constraint_Print( multC2 ) );
    Strings.Add( Constraint_Print( addC ) );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );

    Strings.Add( 'Changing celcius to 0:' );
    celcius.SetValue( 0 );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );

    Strings.Add( 'Changing fahrenheit to 212:' );
    fahrenheit.SetValue( 212 );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );

    Strings.Add( 'Changing celcius to -40:' );
    celcius.SetValue( -40 );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );

    Strings.Add( 'Changing fahrenheit to 70:' );
    fahrenheit.SetValue( 70 );
    Strings.Add( Variable_Print( celcius )
                 + ' = ' +
                 Variable_Print( fahrenheit ) );
    Strings.Add( '' );
(*

    printf("Changing celcius to 0:\n  ");
    Assign(celcius, 0);
    Variable_Print(celcius); printf(" = ");
    Variable_Print(fahrenheit); printf("\n\n");

    printf("Changing fahrenheit to 212:\n  ");
    Assign(fahrenheit, 212);
    Variable_Print(celcius); printf(" = ");
    Variable_Print(fahrenheit); printf("\n\n");

    printf("Changing celcius to -40:\n  ");
    Assign(celcius, -40);
    Variable_Print(celcius); printf(" = ");
    Variable_Print(fahrenheit); printf("\n\n");

    printf("Changing fahrenheit to 70:\n  ");
    Assign(fahrenheit, 70);
    Variable_Print(celcius); printf(" = ");
    Variable_Print(fahrenheit); printf("\n\n");
*)
  finally
    engine.Free;
  end;
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  TempConverter( Memo1.Lines );
end;

end.
